<?php
/*********************************************************************************
 * The contents of this file are subject to the TimeTrex Public License Version
 * 1.1.0 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.TimeTrex.com/TPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by TimeTrex" logo and
 *    (ii) the TimeTrex copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: TimeTrex Open Source
 * The Initial Developer of the Original Code is TimeTrex Payroll Services
 * Portions created by TimeTrex are Copyright (C) 2004-2007 TimeTrex Payroll Services;
 * All Rights Reserved.
 *
 ********************************************************************************/
/*
 * $Revision: 417 $
 * $Id: ClientBalanceFactory.class.php 417 2006-12-06 22:58:53Z ipso $
 * $Date: 2006-12-06 14:58:53 -0800 (Wed, 06 Dec 2006) $
 */
class ClientBalanceFactory extends Factory {
	protected $table = 'client_balance';

	function getBalance() {
		if ( isset($this->data['balance']) ) {
			return Misc::MoneyFormat($this->data['balance'], FALSE );
		}

		return FALSE;
	}

	function initializeBalance( $client_id ) {
		if ( $client_id == '' ) {
			return FALSE;
		}

		global $current_user;

		if ( is_object($current_user) ) {
			$updated_by = $current_user->getID();
		} else {
			$updated_by = 0;
		}

		$cblf = new ClientBalanceListFactory();
		$cblf->getByClientId( $client_id );
		if ( $cblf->getRecordCount() == 0 ) {
			$ph = array(
						'client_id' => $client_id,
						'current_date' => TTDate::getTime(),
						'updated_by' => $updated_by,
						);
			$query = 'insert into '. $this->getTable() .' (client_id,balance,updated_date, updated_by) VALUES(?,\'0.00\',?,? )';
			return $this->db->Execute($query, $ph);
		}

		return TRUE;
	}

	function modifyBalance( $client_id, $company_id, $amount ) {
		if ( $client_id == '' ) {
			$this->FailTransaction();
			return FALSE;
		}

		if ( $company_id == '' ) {
			$this->FailTransaction();
			return FALSE;
		}

		if ( $amount == 0 ) {
			Debug::Text('Amount is 0, skipping: '. $amount, __FILE__, __LINE__, __METHOD__,10);
			return TRUE;
		}

		if ( !is_numeric($amount) ) {
			Debug::Text('FAILED on Amount: '. $amount, __FILE__, __LINE__, __METHOD__,10);
			$this->FailTransaction();
			return FALSE;
		}

		global $current_user;

		if ( is_object($current_user) ) {
			$updated_by = $current_user->getID();
		} else {
			$updated_by = 0;
		}

		$cf = new ClientFactory();

		$this->StartTransaction();

		$ph = array(
					'company_id' => $company_id,
					'client_id' => $client_id,
					);

		$query = 'select a.client_id, a.balance
						from '. $this->getTable() .' as a, '. $cf->getTable() .' as b
						where a.client_id = b.id
							AND b.company_id = ?
							AND a.client_id = ?
						FOR UPDATE';
		$row = $this->db->GetRow($query, $ph);
		$balance = Misc::MoneyFormat( $row['balance'] + $amount, FALSE );
		Debug::Text('Amount: '. $amount .' New Balance: '. $balance, __FILE__, __LINE__, __METHOD__,10);

		$tmp_client_id = $row['client_id'];

		$ph = array(
					'balance' => $balance,
					'updated_date' => TTDate::getTime(),
					'updated_by' => $updated_by,
					'client_id' => $tmp_client_id,
					);


		$query = 'update '. $this->getTable() .'
					set 	balance = ?,
							updated_date = ?,
							updated_by = ?
					where client_id = ?';
		$retval = $this->db->Execute($query, $ph);

		$this->CommitTransaction();

		return $retval;
	}

	function deleteBalance( $client_id, $company_id ) {
		if ( $client_id == '' ) {
			return FALSE;
		}

		if ( $company_id == '' ) {
			return FALSE;
		}

		$cblf = new ClientBalanceListFactory();
		$cblf->getByClientIdAndCompanyId( $client_id, $company_id );
		if ( $cblf->getRecordCount() > 0 ) {
			//No 'id' column on this table, so we have to delete manually.
			$ph = array( 'client_id' => $client_id );

			$query = 'delete from '. $this->getTable() .' where client_id = ?';
			$retval = $this->db->Execute($query, $ph);
		}

		return TRUE;
	}

	function reCalculateBalance( $client_id, $company_id ) {
		if ( $client_id == '' ) {
			return FALSE;
		}

		if ( $company_id == '' ) {
			return FALSE;
		}

		$this->StartTransaction();

		$this->deleteBalance( $client_id, $company_id );
		$this->initializeBalance( $client_id );

		$tlf = new TransactionListFactory();
		$tlf->getByClientIdAndCompanyId( $client_id, $company_id );
		if ( $tlf->getRecordCount() > 0 ) {
			foreach( $tlf as $t_obj ) {
				if ( $t_obj->getStatus() == 10 ) { //Confirmed
					$this->modifyBalance( $client_id, $company_id, $t_obj->getSignedAmount() );
				}
			}
		}

		$this->CommitTransaction();

		return TRUE;
	}

	//This table doesn't have any of these columns, so overload the functions.
	function getDeleted() {
		return FALSE;
	}
	function setDeleted($bool) {
		return FALSE;
	}

	function getCreatedDate() {
		return FALSE;
	}
	function setCreatedDate($epoch = NULL) {
		return FALSE;
	}
	function getCreatedBy() {
		return FALSE;
	}
	function setCreatedBy($id = NULL) {
		return FALSE;
	}

	function getDeletedDate() {
		return FALSE;
	}
	function setDeletedDate($epoch = NULL) {
		return FALSE;
	}
	function getDeletedBy() {
		return FALSE;
	}
	function setDeletedBy($id = NULL) {
		return FALSE;
	}
}
?>
